use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
AtomicFile
BUF
Chase
Dianne
Dorfman
Doru
Erik
FOO
Foo
IO
InnerFile
Lines
NBYTES
POS
SCALARREF
SLAVECLASS
Scalar
ScalarArray
Skoll
Stringy
Whitener
Wrap
WrapTie
ZeeGee
aref
capoeirab
dfs
dskoll
eryq
getline
getlines
getpos
ing
lib
reblessed
setpos
sref
tieable
wraphandle
